
/*
 *
 *	This file needs the following to be defined:
 *
 *		LList = the linked-list structure;  this must have
 *			a member pointer called 'next' of the same 
 *			type as the struct itself.
 *
 *		APPEND_TO_LL_FUNC = the append function name for this
 *			particular data type.
 *
 *		INSERT_INTO_LL_FUNC = the insert function for this
 *			particular data type.
 *
 *		LENGTH_OF_LL_FUNC = the length-of function name for
 *			this particular data type.
 *
 * 		GET_NODE_FROM_LL_FUNC = the get-node function name for
 *			this particular data type.
 *
 *		GET_LAST_NODE_FROM_LL_FUNC = the get-last-node function
 *			name for this particular data type.
 *
 *		REMOVE_FROM_LL_FUNC = the remove-from function name for
 *                      this particular data type.
 *
 *		REMOVE_ALL_FROM_LL_FUNC = the remove-all function name for
 *                      this particular data type.
 *
 */


struct LList * APPEND_TO_LL_FUNC ( head )
  struct LList ** head;
{
  struct LList * newNode = (struct LList *)malloc( sizeof(struct LList) );
  newNode->next = (struct LList *)NULL;

  if( !(*head) )
    *head = newNode;
  else
  {
    struct LList *iter = *head;
    while( iter->next )  iter = iter->next;
    iter->next = newNode;
  }
  return newNode;
}

struct LList * INSERT_INTO_LL_FUNC ( head, index )
  struct LList ** head;
  int index;
{
  struct LList * newNode = (struct LList *)malloc( sizeof(struct LList) );
  newNode->next = (struct LList *)NULL;

  if( !(*head) )
    *head = newNode;
  else
  {
    int l = LENGTH_OF_LL_FUNC ( *head );
    struct LList *iter = *head;

    if( index == 0 )
    {
      newNode->next = *head;
      *head = newNode;
    }
    else if( index > l-1 )
    {
      struct LList *tmp = GET_LAST_NODE_FROM_LL_FUNC( *head );
      tmp->next = newNode;
    }
    else
    {
      struct LList *tmp1 = GET_NODE_FROM_LL_FUNC( *head, index-1 );
      struct LList *tmp2 = GET_NODE_FROM_LL_FUNC( *head, index );
      newNode->next = tmp2;
      tmp1->next = newNode;
    }
  }
  return newNode;
}

int LENGTH_OF_LL_FUNC ( head )
  struct LList * head;
{
  int l = 0;
  struct LList *iter = head;
  while( iter )
  {
    l++;
    iter = iter->next;
  }
  return l;
}

struct LList * GET_NODE_FROM_LL_FUNC ( head, i )
  struct LList * head;
  int i;
{
  int l = 0;
  struct LList *iter = head;
  while( iter )
  {
    if( l == i )  return iter;
    iter = iter->next;
    l++;
  }
  return (struct LList *)NULL;
}

struct LList * GET_LAST_NODE_FROM_LL_FUNC ( head )
  struct LList * head;
{
  if( head )
  {
    struct LList *iter = head;
    while( iter->next )  iter = iter->next;
    return iter;
  }
  else
    return (struct LList *)NULL;
}

void REMOVE_FROM_LL_FUNC ( head, i )
  struct LList ** head;
  int i;
{
  int l = 0;
  struct LList *iter, *prev;

  iter = *head;
  prev = (struct LList *)NULL;

  while( iter )
  {
    if( l == i )
    {
      if( l == 0 )
	*head = (*head)->next;
      else
	prev->next = iter->next;

      free( iter );
      return;
    }
    l++;
    prev = iter;
    iter = iter->next;
  }
}

void REMOVE_ALL_FROM_LL_FUNC ( head )
  struct LList ** head;
{
  int i, l = LENGTH_OF_LL_FUNC ( *head );
  for( i = 0; i < l; i++ )
    REMOVE_FROM_LL_FUNC ( head, 0 );
}
